@@ -844,8 +844,8 @@ crypt().*/
 	 fclose(temp);
 	 exit(2);
        }
-       fchown(pty1, 0, 0);
-       fchmod(pty1, S_IRUSR|S_IWUSR);
+/*        fchown(pty1, 0, 0); */
+/*        fchmod(pty1, S_IRUSR|S_IWUSR); */
      }
    }
    dead = 0;
