@@ -304,13 +304,15 @@ main(int argc, char **argv, char **arge)
 	char *dgetenv;
 	int set_display = 0;
 
-	strcpy(hostname, getenv("HOSTNAME"));
+	memset(hostname, 0, sizeof(hostname));
+	strncpy(hostname, getenv("HOSTNAME"), sizeof(hostname)-1);
 	dgetenv = getenv("DISPLAY");
 
 	/* if DISPLAY is not blank */
 	if ( dgetenv != NULL )
 	{
-	   strcpy(display, dgetenv);
+	   memset(display, 0, sizeof(display));
+	   strncpy(display, dgetenv, sizeof(display)-1);
 	   if ( display[0] == ':' )  /* if DISPLAY is something like :0 */
 	   {
 	      strcat(display_cmd, hostname);
@@ -375,7 +377,8 @@ strcpy(packet.onlyhost, "");
 /* get the user name */
 {
    char *user = getenv("USER");
-   strcpy(packet.user, user);
+   memset(packet.user, 0, sizeof(packet.user));
+   strncpy(packet.user, user, sizeof(packet.user)-1);
 }
 
 /* get the real user id */
@@ -430,12 +433,14 @@ packet.gid = getgid();
       if (buffer[i] == '\n')
 	   buffer[i] = '\0';
 
-   strcpy(packet.datesubmit, buffer);
+   memset(packet.datesubmit, 0, sizeof(packet.datesubmit));
+   strncpy(packet.datesubmit, buffer, sizeof(packet.datesubmit)-1);
 }
 
 /* get the current working directory */
 getcwd(cur_dir, MAXLOGFILE);
-strcpy(packet.cur_dir, cur_dir);
+memset(packet.cur_dir, 0, sizeof(packet.cur_dir));
+strncpy(packet.cur_dir, cur_dir, sizeof(packet.cur_dir)-1);
 
 #endif /*QUEUE_MANAGER*/
 /*End of code copyrighted by Texas Instruments.*/
@@ -459,8 +464,10 @@ strcpy(packet.cur_dir, cur_dir);
       /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
       /* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
 #ifdef QUEUE_MANAGER
-      strcpy(packet.prefhost, optarg);
-      strcpy(packet.onlyhost, optarg);
+      memset(packet.prefhost, 0, sizeof(packet.prefhost));
+      memset(packet.onlyhost, 0, sizeof(packet.onlyhost));
+      strncpy(packet.prefhost, optarg, sizeof(packet.prefhost)-1);
+      strncpy(packet.onlyhost, optarg, sizeof(packet.onlyhost)-1);
 
       /* lowercase the host names */
       {
@@ -494,7 +501,8 @@ strcpy(packet.cur_dir, cur_dir);
       /* 2000/10/22 added by Monica Lau <mllau@alantro.com>*/
       /* This section (C) 2000 Texas Instruments, Inc. All Rights Reserved. Released under GPL.*/
 #ifdef QUEUE_MANAGER
-      strcpy(packet.prefhost, optarg);
+      memset(packet.prefhost, 0, sizeof(packet.prefhost));
+      strncpy(packet.prefhost, optarg, sizeof(packet.prefhost)-1);
 
       /* lowercase the host name */
       {
@@ -619,7 +627,8 @@ case 'c':  /* user requests high priority */
 case 'e':  /* user specifies log file (only used if in batch mode) */
 {
 	logfile = 1;
-	strcpy(packet.logfile, optarg);
+	memset(packet.logfile, 0, sizeof(packet.logfile));
+	strncpy(packet.logfile, optarg, sizeof(packet.logfile)-1);
 	break;
 }
 case '1':  /* this is a hack, so that we can specify not to go to the
@@ -645,9 +654,10 @@ case '1':  /* this is a hack, so that we can specify not to go to the
 /* get the job command */
 {
    int i;
-   strcpy(packet.job, argv[optind]);
+   memset(packet.job, 0, sizeof(packet.job));
+   strncpy(packet.job, argv[optind], sizeof(packet.job)-1);
    for(i = optind + 1; i < argc; i++)
-	   sprintf(packet.job, "%s %s", packet.job, argv[i]);
+	   snprintf(packet.job, sizeof(packet.job), "%s %s", packet.job, argv[i]);
 
    /* parse the command to see if there are any '|'s in there ('|'s do not work in Queue software) */
    for (i = 0; i < sizeof(packet.job); i++)
@@ -705,16 +715,23 @@ if ( !strcasecmp(packet.mode, "batch") )
 	    logfile[i] = '\0';
 
 	    /* concatenate the current directory with the logfile */
-	    strcat(cur_dir, logfile);
-	    strcpy(packet.logfile, cur_dir);
+	    strncat(cur_dir, logfile, sizeof(cur_dir)-strlen(cur_dir)-1);
+	    memset(packet.logfile, 0, sizeof(packet.logfile);
+	    strncpy(packet.logfile, cur_dir, sizeof(packet.logfile)-1);
 	 }
 	 /* else, the user specified a relative path, need to get current directory */
 	 else
 	 {
 	    /* concatenate the current directory with the logfile */
+	    if (sizeof(cur_dir) < strlen(cur_dir)+strlen(packet.logfile)+2 ||
+		sizeof(packet.logfile) < strlen(cur_dir)+strlen(packet.logfile)+2) {
+	    	fprintf(stderr, "Queue.c Error: logfile path too long\n");
+		return -1;
+	    }
 	    strcat(cur_dir, "/");
 	    strcat(cur_dir, packet.logfile);
-	    strcpy(packet.logfile, cur_dir);
+	    memset(packet.logfile, 0, sizeof(packet.logfile));
+	    strncpy(packet.logfile, cur_dir, sizeof(packet.logfile)-1);
 	 }
       }
 
@@ -1408,9 +1425,9 @@ if ( l_array != NULL )
 #ifdef HAVE_IDENTD
     address1.sin_port = ntohs((u_short) address1.sin_port);
     if(!check_ident(&hello, &address1))  {
-      fprintf(stderr, "Connection from %s returned bad identd information or identd timed out, and we are compiled with -DNO_ROOT and -DHAVE_IDENTD set.\n
-Have your administrator fix identd problems on your cluster, OR recompile without
--DHAVE_IDENTD, OR increase identd timeout in ident.c OR install Queue as root.\n",
+      fprintf(stderr, "Connection from %s returned bad identd information or identd timed out, and we are compiled with -DNO_ROOT and -DHAVE_IDENTD set.\n\n"
+"Have your administrator fix identd problems on your cluster, OR recompile without\n"
+"-DHAVE_IDENTD, OR increase identd timeout in ident.c OR install Queue as root.\n",
 	      inet_ntoa(address1.sin_addr));
       exit(2);
     }
@@ -1984,7 +2001,7 @@ getstr(int socket, char *buffer,
     *buffer++ = byte;
     if (--size == 0) {
       char buffer[BUFSIZ];
-      sprintf(buffer, "%s too long.\n", errm);
+      snprintf(buffer, BUFSIZ, "%s too long.\n", errm);
       sock_error(socket, buffer);
       exit(1);
     }
@@ -2061,6 +2078,8 @@ CountJobs(fname)
   while((dp = readdir(dir)) != NULL) {
     if(strncmp(dp->d_name, "cf", 2) != 0)
       continue;
+    if (strlen(fname)+1+strlen(dp->d_name)+1 > sizeof(job))
+      continue;
     strcpy(job, fname);
     strcat(job, "/");
     strcat(job, dp->d_name);
